#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="TrashRemove" trashdelay="10s"/>
        </fs>
        <fs type="LocalFilesystem" path="${MYDIR}s" name="slave">
            <aspect type="TrashRemove" trashdelay="10s"/>
        </fs>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
    </sync>
</parsleyconfig>
        """)

        write2file("s/_dummy", "")

        write2file("m/x", "x")
        SYNC()
        deletefile("s/x")
        link("x", "s/x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        self.assertTrue(ll[0] in ["master", "slave"])
        self.assertTrue(ll[1] in ["master", "slave"])

        write2file("m/.parsley.control/conflicts/test/x", "master")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(readfile("m/x") == "x")
        self.assertTrue(readfile("s/x") == "x")
        SYNC()

        # file > link
        deletefile("s/x")
        link("x", "s/x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "slave")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(islink("m/x") == "x")
        self.assertTrue(islink("s/x") == "x")
        SYNC()

        # link > dir
        deletefile("s/x")
        write2file("s/x/foo", "x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "slave")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(readfile("m/x/foo") == "x")
        self.assertTrue(readfile("s/x/foo") == "x")

        # dir > link
        deletedir("m/x")
        link("x", "m/x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "master")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(islink("m/x") == "x")
        self.assertTrue(islink("s/x") == "x")
        SYNC()

        # link > file
        deletefile("m/x")
        write2file("m/x", "x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "master")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(readfile("m/x") == "x")
        self.assertTrue(readfile("s/x") == "x")
        SYNC()

        # file > dir
        deletefile("s/x")
        write2file("s/x/foo", "x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "slave")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(readfile("m/x/foo") == "x")
        self.assertTrue(readfile("s/x/foo") == "x")

        # dir > file
        deletedir("m/x")
        write2file("m/x", "x")
        SYNC()
        self.assertTrue(haslogentry("x", verb="conflict"))
        ll = readfile("m/.parsley.control/conflicts/test/x", readall=True).split("\n")
        self.assertTrue(len(ll) == 2)
        write2file("m/.parsley.control/conflicts/test/x", "master")
        SYNC()
        self.assertTrue(not fileexists("m/.parsley.control/conflicts/test/x"))
        self.assertTrue(not fileexists("s/.parsley.control/conflicts/test/x"))
        self.assertTrue(readfile("m/x") == "x")
        self.assertTrue(readfile("s/x") == "x")
        SYNC()


if __name__ == '__main__':
    unittest.main()
