#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="TrashRemove"/>
        </fs>
        <fs type="LocalFilesystem" path="${MYDIR}s" name="slave">
            <aspect type="TrashRemove"/>
        </fs>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
    </sync>
</parsleyconfig>
        """)

        write2file("s/d1/d2/d3/d4/d5/f1", "f1")
        write2file("s/e1/e2/e3/e4/e5/f2", "f2")
        write2file("m/_dummy", "")  # otherwise the other dir gets empty which is a bad thing for the watchdog
        SYNC()
        time.sleep(1)
        self.assertTrue(readfile("s/d1/d2/d3/d4/d5/f1") == "f1")
        self.assertTrue(readfile("m/e1/e2/e3/e4/e5/f2") == "f2")
        write2file("s/d1/d2/f3", "f3")
        write2file("m/e1/e2/f4", "f4")
        deletefile("s/d1/d2/d3/d4/d5/f1")
        deletefile("s/e1/e2/e3/e4/e5/f2")
        deletedir("s/d1/d2/d3/d4/d5/")
        deletedir("s/d1/d2/d3/d4")
        deletedir("s/d1/d2/d3")
        deletedir("s/e1/e2/e3/e4/e5/")
        deletedir("s/e1/e2/e3/e4/")
        deletedir("s/e1/e2/e3/")
        deletedir("s/e1/e2/")
        deletedir("s/e1/")
        SYNC()
        time.sleep(1)
        SYNC()
        time.sleep(1)
        SYNC()
        self.assertTrue(not isdir("m/d1/d2/d3/"))
        self.assertTrue(readfile("m/e1/e2/f4") == "f4")
        self.assertTrue(readfile("m/d1/d2/f3") == "f3")
        self.assertTrue(readfile("s/e1/e2/f4") == "f4")
        self.assertTrue(readfile("s/d1/d2/f3") == "f3")
        self.assertTrue(listdir("m/d1/d2") == ["f3"])


if __name__ == '__main__':
    unittest.main()
