#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="TrashRemove" trashdelay="10s"/>
        </fs>
        <fs type="LocalFilesystem" path="${MYDIR}s" name="slave">
            <aspect type="TrashRemove" trashdelay="10s"/>
        </fs>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
        <preparation type="MountPreparation" src="${MYDIR}part" tgt="${MYDIR}m" options_0="-o" options_1="loop"/>
    </sync>
</parsleyconfig>
        """)

        os.makedirs(mydir+"s")
        os.makedirs(mydir+"m")
        subprocess.call(["dd", "if=/dev/zero", "of="+mydir+"part", "bs=4096", "count=1000"])
        subprocess.call(["dd", "if=/dev/zero", "of="+mydir+"s/file", "bs=4096", "count=1250"])
        subprocess.call(["mkfs.ext3", "-F", mydir+"part"])
        subprocess.call(["sudo", "-A", "mount", "-o", "loop", mydir+"part", mydir+"m"])
        time.sleep(1)
        subprocess.call(["sudo", "-A", "chmod", "-R", "777", mydir+"m"])
        time.sleep(1)
        subprocess.call(["sudo", "-A", "umount", mydir+"m"])
        time.sleep(2)
        try:
            SYNC()
        except Exception:
            pass
        subprocess.call(["sudo", "-A", "mount", "-o", "loop", mydir+"part", mydir+"m"])
        try:
            self.assertTrue(fileexists("m/.parsley.control/temp/_parsley_currenttransfer.1"))
            self.assertTrue(readfile("m/file") == "")
            time.sleep(2)
            subprocess.call(["sudo", "-A", "umount", mydir+"m"])
            time.sleep(2)
            deletefile("s/file")
            try:
                SYNC()
            except Exception:
                pass
            subprocess.call(["sudo", "-A", "mount", "-o", "loop", mydir+"part", mydir+"m"])
            self.assertFalse(fileexists("m/.parsley.control/temp/_parsley_currenttransfer.1"))
            self.assertTrue(readfile("m/file") == "")
            write2file("s/file", "file")
            time.sleep(2)
            subprocess.call(["sudo", "-A", "umount", mydir+"m"])
            time.sleep(2)
            try:
                SYNC()
            except Exception:
                pass
            subprocess.call(["sudo", "-A", "mount", "-o", "loop", mydir+"part", mydir+"m"])
            self.assertTrue(readfile("m/file") == "file")
            time.sleep(2)
        finally:
            subprocess.call(["sudo", "-A", "umount", mydir+"m"])


if __name__ == '__main__':
    unittest.main()
