#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="PullAndPurgeSyncSink"/>
            <aspect type="MetadataSynchronizationWithShadow"/>
        </fs>
        <fs type="SshfsFilesystem" path="${MYDIR}s" sshtarget="pino@localhost" idfile="${MYDIR}loginkey" name="slave"
            port="42921"
            options_0="-o" options_1="StrictHostKeyChecking=no" options_2="-o" options_3="UserKnownHostsFile=/dev/null">
            <aspect type="PullAndPurgeSyncSource"/>
            <aspect type="MetadataSynchronization"/>
        </fs>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
    </sync>
</parsleyconfig>
        """)

        with TemporarySshFs(mydir):
            os.makedirs(mydir + "s/.parsley.control")

            write2file("s/d1/d2/f1", "f1")
            write2file("s/d1/f2", "f2")
            write2file("m/_dummy", "")
            metadata.setfilemetadata("s/d1/d2/f1", "foo", "1")
            SYNC()

            metadata.setfilemetadata("m/d1/f2", "foo", "2")
            SYNC()

            self.assertTrue(metadata.getfilemetadata("m/d1/d2/f1", "foo") == "1")
            self.assertTrue(metadata.getfilemetadata("m/d1/f2", "foo") == "2")
            self.assertTrue(metadata.getshadowmetadata("m/d1/d2/f1", "foo") == "1")
            self.assertTrue(metadata.getshadowmetadata("m/d1/f2", "foo") == "2")

            metadata.setshadowmetadata("m/d1/f2", "foo", "22")
            metadata.killshadowmetadata("m/d1/d2/f1")

            SYNC()

            self.assertTrue(metadata.getfilemetadata("m/d1/f2", "foo") == "22")
            self.assertTrue(metadata.getshadowmetadata("m/d1/f2", "foo") == "22")
            self.assertTrue(metadata.getfilemetadata("m/d1/d2/f1", "foo") == "1")
            self.assertTrue(metadata.getshadowmetadata("m/d1/d2/f1", "foo") == "1")


if __name__ == '__main__':
    unittest.main()
