#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="DefaultRemove"/>
            <aspect type="MetadataSynchronizationWithShadow"/>
        </fs>
        <fs type="SshfsFilesystem" path="${MYDIR}s" sshtarget="pino@localhost" idfile="${MYDIR}loginkey" name="slave"
            port="42921" timeout="4s"
            options_0="-o" options_1="StrictHostKeyChecking=no" options_2="-o" options_3="UserKnownHostsFile=/dev/null">
            <aspect type="TrashRemove"/>
            <aspect type="MetadataSynchronization"/>
            <aspect type="SleepWhileBeginning"/>
        </fs>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
    </sync>
    <customaspect name="SleepWhileBeginning">
import random, subprocess, shutil, signal, random, threading, datetime
import parsley.syncengine.sync
class SleepWhileBeginning(parsley.aspect.abstractaspect.Aspect):
    def __init__(self):
        parsley.aspect.abstractaspect.Aspect.__init__(self)
    @parsley.aspect.hook("", "", "", event=parsley.syncengine.common.SyncEvent.UpdateDir_Prepare)
    def sleepwhilebeginupdatedir(self, ctx, filesystem):
        if ctx.path == "":
            if os.path.exists("/var/tmp/_parsley_ctrl_die_in_beginning"):
                os.unlink("/var/tmp/_parsley_ctrl_die_in_beginning")
                time.sleep(3)
    </customaspect>
</parsleyconfig>
        """)

        with TemporarySshFs(mydir) as tempssh:
            os.makedirs(mydir + "s/.parsley.control")

            try:
                os.unlink("/var/tmp/_parsley_ctrl_die_in_beginning")
            except OSError:
                pass

            def wait_and_kill_connection():
                with open("/var/tmp/_parsley_ctrl_die_in_beginning", "w"):
                    pass
                while os.path.exists("/var/tmp/_parsley_ctrl_die_in_beginning"):
                    pass
                dis = Disturbator(0, tempssh, mydir)
                dis.d_killmounter()

            write2file("s/_dummy", "")
            write2file("m/a", "a")

            SYNC()

            write2file("m/b", "b")

            threading.Thread(target=wait_and_kill_connection).start()

            try:
                SYNC()
            except Exception:
                pass

            self.assertTrue(readfile("m/a") == "a")
            self.assertTrue(readfile("s/a") == "a")
            self.assertTrue(readfile("m/b") == "b")
            self.assertTrue(not fileexists("s/b"))

            SYNC()

            self.assertTrue(readfile("m/a") == "a")
            self.assertTrue(readfile("s/a") == "a")
            self.assertTrue(readfile("m/b") == "b")
            self.assertTrue(readfile("s/b") == "b")


if __name__ == '__main__':
    unittest.main()
