#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master" external_control_directory="${MYDIR}ctrl">
        </fs>
        <fs type="LocalFilesystem" path="${MYDIR}s" name="slave">
            <aspect type="DefaultRemove"/>
        </fs>
        <aspect type="MySetMtimeGranularity"/>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
    </sync>
    <customaspect name="MySetMtimeGranularity">
import parsley.syncengine.sync
class MySetMtimeGranularity(parsley.aspect.abstractaspect.Aspect):
    def __init__(self):
        parsley.aspect.abstractaspect.Aspect.__init__(self)
    @parsley.aspect.hook("", "", "", event=parsley.syncengine.common.SyncEvent.BeginSync)
    def mysetmtimegranularity(self, ctx, filesystem):
        filesystem._is_mtime_precision_fine = True
    </customaspect>
</parsleyconfig>
        """)

        createdir("m")
        createdir("_m")
        subprocess.call(["sudo", "-A", "mount", "--bind", "-o", "ro", mydir+"_m", mydir+"m"])
        try:
            isreadonly = False
            try:
                write2file("m/foo", "foo")
            except:
                isreadonly = True
            self.assertTrue(isreadonly)
            write2file("_m/f1", "x")
            write2file("_m/f2", "x")
            createdir("s")
            SYNC()
            self.assertTrue(readfile("s/f1") == "x")
            self.assertTrue(readfile("s/f2") == "x")
            deletefile("_m/f1")
            write2file("_m/f2", "y")
            write2file("_m/f3", "y")
            SYNC()
            self.assertTrue(not fileexists("s/f1"))
            self.assertTrue(readfile("s/f2") == "y")
            self.assertTrue(readfile("s/f3") == "y")
            self.assertTrue(fileexists("ctrl/README"))
        finally:
            subprocess.call(["sudo", "-A", "umount", mydir+"m"])


if __name__ == '__main__':
    unittest.main()
