#!/usr/bin/python3
# Copyright (C) 2014-2021, Josef Hahn
#
# This file is part of parsley.
#
# parsley is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# parsley is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with parsley.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
sys.path.append(os.path.abspath(__file__+"/../../.."))
from parsleytest.test import *


class Test(unittest.TestCase):

    def test_main(self):

        setconfig("""<?xml version="1.0" ?>
<parsleyconfig>
    <sync name="test" interval="0s">
        <fs type="LocalFilesystem" path="${MYDIR}m" name="master">
            <aspect type="TrashRemove" />
        </fs>
        <fs type="LocalFilesystem" path="${MYDIR}_s" name="slave">
            <aspect type="TrashRemove" />
        </fs>
        <aspect type="DefaultSync"/>
        <aspect type="Logging" logupdate="1" logcreate="1" logremove="1" />
        <preparation type="SshfsMountPreparation" src="pino@localhost:${MYDIR}s" tgt="${MYDIR}_s"
            idfile="${MYDIR}loginkey" port="42921"
            options_0="-o" options_1="StrictHostKeyChecking=no" options_2="-o" 
            options_3="UserKnownHostsFile=/dev/null"/>
    </sync>
</parsleyconfig>
        """)

        with TemporarySshFs(mydir):
            os.mkdir(mydir + "_s")

            write2file("m/d1/d2/f1", "f1")
            write2file("m/d1/d2/f2", "f2")
            write2file("m/d1/f3", "f3")
            write2file("s/d5/d7/f4", "f4")
            write2file("s/d5/d6/f5", "f5")
            write2file("s/d1/d2/f6", "f6")

            SYNC()

            self.assertTrue(readfile("m/d1/d2/f2") == "f2")
            self.assertTrue(readfile("m/d1/d2/f2") == "f2")
            self.assertTrue(readfile("m/d1/f3") == "f3")
            self.assertTrue(readfile("s/d1/d2/f1") == "f1")
            self.assertTrue(readfile("s/d1/d2/f2") == "f2")
            self.assertTrue(readfile("s/d1/f3") == "f3")
            self.assertTrue(readfile("m/d5/d7/f4") == "f4")
            self.assertTrue(readfile("m/d5/d6/f5") == "f5")
            self.assertTrue(readfile("m/d1/d2/f6") == "f6")
            self.assertTrue(readfile("s/d5/d7/f4") == "f4")
            self.assertTrue(readfile("s/d5/d6/f5") == "f5")
            self.assertTrue(readfile("s/d1/d2/f6") == "f6")


if __name__ == '__main__':
    unittest.main()
