# SPDX-FileCopyrightText: © 2014 Josef Hahn
# SPDX-License-Identifier: AGPL-3.0-only

import parsley.aspect.abstractaspect
import parsley.config.configpiece
import parsley.syncengine.common


class ApplyPathAcceptor(parsley.aspect.abstractaspect.Aspect):
    """
    Filters out files/dirs that should be excluded from syncing.
    It works by providing a path acceptor function.
    """

    # noinspection PyShadowingBuiltins
    def __init__(self, function):
        """
        :param function: The acceptor function. it gets the path as parameter and must return `True` for accepting it.
                         Warning: This parameter is a string that will be evaluated as Python expression (so
                         usage from config files is easy)! The path argument is available as `path`. 
                         Example: `path[2]=='a'`. The filesystem is available as `fs`.
        """
        super().__init__()
        self.acceptor = parsley.config.configpiece.getcallable(function, ["path", "fs"])

    @parsley.aspect.hook("", "applypathacceptor", "collectinfo",
                         event=parsley.syncengine.common.SyncEvent.UpdateItem_BeforeElectMaster)
    @parsley.aspect.execute_only_if_not_already_maximally_elected()
    def applypathacceptor_applypathacceptor(self, ctx, filesystem):
        """
        Skips this entry if it is rejected by the path acceptor.
        """
        if not self.acceptor(path=ctx.path, fs=filesystem):
            ctx.skip_master_filesystem_promotion()
