# SPDX-FileCopyrightText: © 2014 Josef Hahn
# SPDX-License-Identifier: AGPL-3.0-only

import parsley.aspect.abstractaspect
import parsley.syncengine.common


class CollectInformation(parsley.aspect.abstractaspect.Aspect):
    """
    Used for populating the `._filelists_curr` lists.
    """

    # noinspection PyProtectedMember
    @parsley.aspect.hook("applypathacceptor", "collectinfo", "",
                         event=parsley.syncengine.common.SyncEvent.UpdateItem_BeforeElectMaster)
    @parsley.aspect.execute_only_if_not_already_maximally_elected()
    def collectinformation_collectinfo(self, ctx, filesystem):
        """
        Collects information (type, size, ...) for this entry from the filesystem and stores it to _filelists_curr.
        """
        ftype = filesystem.getftype(ctx.path)
        if ftype:
            if ftype == parsley.syncengine.common.EntryType.Link:
                param = filesystem.getlinktarget(ctx.path)
                size = None
                mtime = None
            else:
                mtime = filesystem.getmtime(ctx.path)
                size = filesystem.getsize(ctx.path)
                param = None
            ctx._filelists_curr[filesystem].foundfile(ctx.path, ftype, size, mtime, param)
        else:
            ctx._filelists_curr[filesystem].notfoundfile(ctx.path)
