# SPDX-FileCopyrightText: © 2014 Josef Hahn
# SPDX-License-Identifier: AGPL-3.0-only

"""
Basic exception classes.
"""


class ParsleyError(Exception):

    def __call__(self, *args):
        return self.__class__(*(self.args + args))


class InvalidCommandLineError(ParsleyError):
    pass


class ConfigurationError(ParsleyError):
    pass


class ReadInvalidConfigurationError(ConfigurationError):
    pass


class ParsleyEngineExecutionError(ParsleyError):
    pass


class PreparationEnabledBeforeEnablingExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Preparation not disabled before enabling")


class PreparationDisabledAfterEnablingExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Preparation disabled after enabling")


class PreparationEnabledAfterDisablingExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Preparation not disabled after disabling")


class ErrorEnablingPreparationExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Unable to enable preparation")


class ErrorDisablingPreparationExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Unable to disable preparation")


class ErrorGettingPreparationStateExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Unable to get preparation state")


class ErrorInitializingExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Unable to initialize the synchronization task")


class ErrorExecutingExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Unable to execute the synchronization task")


class EnablingImpossibleExecutionError(ParsleyEngineExecutionError):

    def __init__(self):
        super().__init__("Impossible to enable the preparation")
