# SPDX-FileCopyrightText: © 2014 Josef Hahn
# SPDX-License-Identifier: AGPL-3.0-only

"""
The Parsley reporting gui.
"""

import os

from parsley.gui.helpers import tr
import parsley.gui.app_report.logpanel
import parsley.gui.app_report.performancepanel
import parsley.gui.app_report.timelinepanel
import parsley.gui.gtk.volumeapp
import parsley.tools.common


_mydir = os.path.dirname(__file__)


class App(parsley.gui.gtk.volumeapp.AbstractVolumeApp):

    def __init__(self):
        super().__init__()
        builder = parsley.gui.gtk.ParsleyGtkBuilder()
        builder.add_from_file(f"{_mydir}/main.ui")
        self.__pnllog = builder.get_object("pnllog")
        self.__pnltimeline = builder.get_object("pnltimeline")
        self.__pnlperformance = builder.get_object("pnlperformance")
        self.__btnrefresh = builder.get_object("btnrefresh")
        self.__wlog = parsley.gui.app_report.logpanel.LogPanel()
        self.__wtimeline = parsley.gui.app_report.timelinepanel.TimelinePanel()
        self.__wperformance = parsley.gui.app_report.performancepanel.PerformancePanel()
        self.__pnllog.add(self.__wlog)
        self.__pnltimeline.add(self.__wtimeline)
        self.__pnlperformance.add(self.__wperformance)
        self._set_body(builder.get_object("mainbody"))
        self._set_application_title(tr("Parsley Report Viewer"))
        self._set_welcome_text(tr("At first, go to the Parsley sync directory you want to see reports for."))
        self._add_action_button(self.__btnrefresh)
        self.__btnrefresh.connect("clicked", lambda *_: self._populate())

    def _has_unsaved_changes(self):
        return False

    def _populate(self):
        super()._populate()
        self.__btnrefresh.props.visible = self._dir_is_valid
        self.__wlog._populate(self._dir)
        self.__wtimeline._populate(self._dir)
        self.__wperformance._populate(self._dir)


def run():
    App().show()
