# SPDX-FileCopyrightText: © 2014 Josef Hahn
# SPDX-License-Identifier: AGPL-3.0-only

"""
Default implementation for path acceptors as e.g. used in
:py:class:`parsley.aspect.applypathacceptor.ApplyPathAcceptor`.
"""

import os
import typing as t

import parsley.tools.common

if t.TYPE_CHECKING:
    import parsley.filesystem.abstractfilesystem


# noinspection PyUnusedLocal
def defaultpathacceptor(path: str, fs: 'parsley.filesystem.abstractfilesystem.Filesystem') -> bool:
    """
    A path acceptor that skips file names with some known 'backup pattern', like ending with `~`.
    """
    bn = os.path.basename(path)
    if bn.endswith("~"):
        return False
    if bn.endswith(".kate-swp"):
        return False
    if bn.startswith(".#"):
        return False
    if bn == "__pycache__":
        return False
    return True


def builtinpathacceptor(path: str, fs: 'parsley.filesystem.abstractfilesystem.Filesystem') -> bool:
    """
    A path acceptor that skips parsley control stuff. It is always used.
    """
    if fs.control_directory.startswith("./"):
        return parsley.tools.common.abspath("///" + path) != parsley.tools.common.abspath("///" + fs.control_directory)
    else:
        return True
